/*

  pg.c

  (c) 1995 BearHeart

  Generic Screen Pager for FreeDOS

  This program was coded particularly for use with my FreeDOS help
  system.  I am making it available as a module for general use because
  it's generally useful.  It is not intended to be the pager-to-beat-
  all-pagers because I don't need that.  If you do, you can either 
  suggest enhancements--and if I think they're really kewl and feel
  like coding them I will--or you can code them yourself and submit
  them to me; but please, if you want me to continue to support this
  code then please don't enhance the code without letting me know.
                      --BearHeart, Phoenix, AZ
                        BearHeart@enet.net or
                        BearHeart@paranoia.com
  
  Programming Environment
  
  This program was written using Dave Dunfield's Micro-C (MCC).  MCC 
  is available as freeware from Dunfield Development Systems (DDS).  
  
  For a copy of MCC, contact Dunfield Development Systems at 
  P.O. Box 31044, Nepean, Ontario (Canada), K2B 8S8.  Or call
  Tel: 613-256-5820  Fax: 613-256-5821  BBS: 613-256-6289
  
  Legal Stuff
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Revision Notes
                  BH 95.02.07  Begin Coding
                  BH 95.03.16  Revisions of memory scheme
                  BH 95.04.04  Read in whole file of line addrs first

*/

#include <stdio.h>
#include "pg.h"

#define MAXSIZELINE (256)   // maximum size of a line
#define MAXSEGMENTS (10)    // max segments in far memory
#define LINESPERSEG (16384)  // longs that will fit in 64k

struct longptr {
  int lsw;
  int msw;
  } longbuf,            // scratch buffer for longs
    *longptr;           // scratch pointer for longs 

unsigned int segments[MAXSEGMENTS]; // array of segment addresses for page buffers

unsigned int numlines;  // number of lines in the file

int Color = false;      // color flag

word VideoRegen;        // video regen pointer
word RegenSize;         // in words 

byte DispMode;          // display adapter mode
byte DispRows;          // display rows (from BIOS)
word DispCols;          // display columns (from BIOS)

byte curatt;            // current attribute

                        // prototypes
void InitVideo();       // call this first
int readlines();        // read in the line addresses
register errorexit();   // exit with a message
register prat();        // print at
register pratf();       // print at with format

char linebuf[MAXSIZELINE];  // buffer for reading lines

#define SEEK_SET (0)               // for fseek()
#define BELL     beep(440, 100)    // 440Hz tone for 100ms
#define CO_STATUS     ATTR(BRIGHT | GREEN, MAGENTA)
#define CO_LO_STATUS  ATTR(GREEN,          MAGENTA)
#define CO_TEXT       ATTR(BRIGHT | CYAN,  BLACK)
#define MO_STATUS     ATTR(BLACK,          WHITE)
#define MO_LO_STATUS  ATTR(GREEN,          WHITE)  // lower contrast 
#define MO_TEXT       ATTR(BRIGHT | WHITE, BLACK)
#define setattr(attr)   (curatt = attr)

byte attr_status, attr_lo_status, attr_text;

FILE *infile;           // input file pointer
char *infilename;       // input file name

main(int argc, char ** argv)
{
if (argc == 1)
  {
  infile = stdin;
  infilename = "<stdin>";
  }
else
  {
  infilename = argv[1];
  infile = fopen(infilename, "r");
  if (infile == NULL)
    errorexit(1, "pg: cannot open file %s\n", infilename);
  }

InitVideo();

if(Color)
  {
  attr_status    = CO_STATUS;
  attr_lo_status = CO_LO_STATUS;
  attr_text      = CO_TEXT;
  }
else
  {
  attr_status    = MO_STATUS;
  attr_lo_status = MO_LO_STATUS;
  attr_text      = MO_TEXT;
  }

setattr(attr_text);

readlines();   // read in all the line addresses
pagefile();   // page thru the file

setattr(attr_text);
prat(DispRows - 1, 0, " ");

exit(0);
}

char *stat[] = 
  { 
  "     ", 
  "<top>", 
  "<end>" 
  };

// readlines()
//
// read the whole file and extract the address of the begining of each
// line.  These addresses are stored in far memory using alloc_seg()
//
// NOTE: This function is MCC specific!
//
readlines()
{
register unsigned int linenum;
int segnum, rc;
unsigned memoffset;

cls();
prat(0, 1, "Reading file . . . ");
for (linenum = 1; rc; linenum++)
  {
  segnum = (linenum - 1) / LINESPERSEG;
  if(!segments[segnum])  // alloc a new space
    {
    if(!(segments[segnum] = (unsigned) alloc_seg(4096)))
      errorexit(2, "pg: Cannot allocate far memory\n");
    }
  ftell(infile, &longbuf.msw, &longbuf.lsw);
  if(!(linenum % 1024))
    pratf(0, 20, "%05d", linenum);
  memoffset = ((linenum % LINESPERSEG) - 1) * 4;
  pokew(segments[segnum], memoffset, longbuf.msw);
  pokew(segments[segnum], memoffset + 2, longbuf.lsw);
  rc = fgets(linebuf, MAXSIZELINE - 1, infile); 
  }
pratf(0, 20, "%05d", linenum);
numlines = linenum - 1;
}

// getline
//
// find a line of text from the line buffer
//
// returns pointer to linebuf or zero if cannot find
//
int getline(unsigned linenum)
{
int segnum;
unsigned memoffset;

segnum = (linenum - 1) / LINESPERSEG;
if(!segments[segnum])
  return 0;
memoffset = ((linenum % LINESPERSEG) - 1) * 4;
longbuf.msw = peekw(segments[segnum], memoffset);
longbuf.lsw = peekw(segments[segnum], memoffset + 2);
if(fseek(infile, longbuf.msw, longbuf.lsw, SEEK_SET))
  return 0;
else
  return fgets(linebuf, MAXSIZELINE - 1, infile);
}

#define statok   0   // s/b enum
#define stattop  1
#define statbot  2

pagefile()
{
register int i;
int lineno;                 // current line number
int topline;                // top line
int screenline;             // 
int statusline;             // line number for status line
int pagesize;               // vert size of display area
int pageoffset;             // 
int kc;                     // current keypress
int rc;                     // return code
char * blanks;              // for a line of blanks
int pgstat;                 // current status for display

pagesize = DispRows - 2;
topline = lineno = 1;
pageoffset = 0;
statusline = DispRows - 1;
rc = -1;
pgstat = statok;
if (blanks = malloc(DispCols + 1))  // build the line of blanks
  for (i = 0; i < DispCols; blanks[i++] = ' ');
else
  errorexit(2, "Cannot allocate memory for blanks\n");
blanks[DispCols] = '\0';
while (kc != ESC)
  {
  setattr(attr_text);
  cls();                       // faster than pading each line with blanks
  setattr(attr_status);
  prat(0, 0, blanks);
  pratf(0, 1, "PG -- %-30s%c", infilename,   // display filename
    strlen(infilename) > 30 ? '+' : ' ');
  if (topline < 1)       // alert top of file?
    {
    topline = 1;
    pgstat = stattop;
    }
  else if (topline > (numlines - pagesize))  // alert bot of file?
    {
    topline = numlines - pagesize;
    pgstat = statbot;
    }
  pratf(0, DispCols - 6, "%5d", topline);   // display top line number 
  lineno = topline;
  while(((lineno - topline) < (DispRows - 2)))  // display loop
    {
    screenline = lineno - topline + 1; // where are we on the screen?
    setattr(attr_text);
    rc = getline(lineno);              // read the next line from input
    if (rc) 
      {
      prat(screenline, 0, linebuf);    // display it
      lineno++;
      }
    else   // past end of file, blank rest of screen
      {
      pgstat = statbot;
      setattr(attr_text);
      for( ; (screenline = lineno - topline) < (DispRows - 2); lineno ++)
        prat(++screenline, 0, blanks);
      }
    }
  setattr(attr_status);
  prat(0, 50, stat[pgstat]);
  if(pgstat)
    BELL;
  pgstat = statok;
  switch(kc = kbget())
    {
    case PG_UP:
      topline -= (pagesize - 1);
      break;
    case PG_DN:
      if((topline + pagesize) < numlines)
        topline = lineno - 1;
      else
        pgstat = statbot;
      break;
    case KEY_HOME:
      topline = 1;
      break;
    case KEY_END:
      topline = numlines - pagesize;
      break;
    case KEY_UP:
      topline--;
      break;
    case KEY_DN:
      if((topline + pagesize) < numlines)
        topline++;
      else
        pgstat = statbot;
      break;
    case ESC:
      break;
    case KEY_LT:
    case KEY_RT:
    case RETURN:
    case SPACE:
    case BK_SPACE:
    default:
      BELL;
      break;
    }
  }
free(blanks);
}

void InitVideo()
{
asm {
  mov ax, BIOS_DATA_SEG
  mov es, ax
  mov al, [es:DISP_MODE]
  mov _DispMode, al
  mov ax, [es:DISP_COLS]
  mov _DispCols, ax
  mov al, [es:DISP_ROWS]         ; actually last row
  inc al
  mov _DispRows, al
  }

RegenSize = DispRows * DispCols;     // for use in asm functions
VideoRegen = DispMode == MONO ? 0xb000 : 0xb800;
Color = DispMode == MONO ? false : true;
}  

cls() 
{
asm {
  mov ax, BIOS_DATA_SEG
  mov es, ax
  xor ax, ax
  mov [es:CURSOR], ax   ; home the cursor
  mov di, ax            ; zero origin for regen
  mov cx, _RegenSize
  mov ax, _VideoRegen
  mov es, ax
  mov ah, _curatt
  mov al, 20h           ; ' '
  rep stosw
  }
}

vidchar(vidrow, vidcol, vidchar, vidattr)
{
asm {
  mov ax, BIOS_DATA_SEG
  mov es, ax
  mov al, 8[bp]        ; col
  mov ah, 10[bp]       ; row
  mov [es:CURSOR], ax
  push ax
  mov al, ah
  xor ah, ah
  mov bx, _DispCols    ; offset = vidrow * DispCols + vidrow
  mul bl
  pop bx
  xor bh, bh           ; add in the col
  add ax, bx
  shl ax, 1            ; * 2 for word based
  mov di, ax           ; zero origin for regen
  mov ax, _VideoRegen
  mov es, ax
  mov ah, 4[bp]        ; attr
  mov al, 6[bp]        ; char
  stosw
  } 
}

register prat(int prow, int pcol, char * s)
{
register int i;

for(i = 0; s[i]; i++)
  {
  vidchar(prow, pcol++, s[i], curatt);
  pcol %= DispCols;     // wrap if necessary
  if(pcol == 0)
    prow++;
  }
}

// NOTE: I wrote the following routines before MCC's _format_ was documented
//       and I haven't had the need to look at them since.  Be forwarned.  
//
register pratf(prow, pcol, unsigned args)
{
unsigned *ptr;
char buffer[256];
int row, col;
register int i;

ptr = ((i = nargs()) - 2) * 2 + &args;
_format_(ptr, buffer);

col = * (int *) ptr;   // ptr points at the next arg
ptr += 2;
row = * (int *) ptr;
prat(row, col, buffer);
}

register errorexit(pcode, unsigned args)
{
unsigned *ptr;
int code;
char buffer[256];

ptr = (nargs() - 1) * 2 + &args;
_format_(ptr, buffer);

code = * (int *) ptr;  // ptr points at the next arg
printf(buffer);
exit(code);
}

