/*

Free-DOS TYPE.C by James Hall
Compiled with MICRO-C v1.13

This program displays a text file or files. Several options
are provided to control presentation of the file.

---------------------------------------------------------------------

This program is a part of the Free-DOS project. For more information
about the Free-DOS project and its goals, see the Free-DOS manifesto, 
shich should have been included with this program, or contact 
freedos@sunsite.unc.edu for more information.

---------------------------------------------------------------------

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

---------------------------------------------------------------------

Revision History:

March 1995: James Hall. 
Original version.

June 1995: M. "Hannibal" Toal. 
Port to Micro-C, cosmetic changes.

*/

#include <stdio.h>
#include "freedos.h"
#include "getopt.c"
#include "pagel.c"

/* Global variables to determine TYPE option defaults */
int i, iNum, iBlank, iCtrl, iEnds, iTabs, iUpc, iDownc, iPage;

void fcat (FILE *p);
int blank (char buf[MAX_STR]);
void usage (void);

main (int argc, char **argv)

{
	FILE *p;

	iNum = 0; iBlank = 0; iCtrl = 0; iEnds = 0; iTabs = 0;
	iUpc = 0; iDownc = 0; iPage = 0;

	/* Scan the command line */
	while ((i = getopt (argc, argv, "NSCETULAP?", NULL)) != EOF)
		{
		switch (i)
			{
			/* Number output lines */
			case 'N':
				iNum = 1;
				break;

			/* Squeeze together non-blank lines */
			case 'S':
				iBlank = 1;
				break;

			/* Display ctrl chars with "^" */
			case 'C':       
				iCtrl = iTabs = 1; 
				break;

			/* Display end-of-lines using '$' */
			case 'E':
				iEnds = 1;
				break;

			/* Display tabs using '^I' */
			case 'T':
				iTabs = 1;
				break;

			/* Convert the file to uppercase */
			case 'U':               
				iUpc = 1;
				iDownc = 0;
				break;

			/* Convert the file to lowercase */
			case 'L':
				iDownc = 1;
				iUpc = 0;
				break;

			/* Show all */
			case 'A':
				iNum = 1; iCtrl = 1; iTabs = 1; iEnds = 1;
				break;

			case 'P':
				iPage = 1;
				break;

			/* Show usage message */
			default:
				usage ();
				break;
			}
		}

	/* Display the files */
	if ((argc - optind) < 1)
		fcat (stdin);

	for (i = optind; i < argc; i++)
		{
		if ((p = fopen (argv[i], "r")) != NULL)
			{
			fcat (p);
			fclose (p);
			}
		else
			fprintf (stderr, "Cannot open file: %s\n", argv[i]);
		}
	exit (0);
}

void fcat (FILE *p)

{
	int i, iLen, iLast, lLines;
	char sz[MAX_STR];

	iLast = 0;

	/* Read the file until EOF */
	for (lLines = 1; fgets (sz, MAX_STR, p) != NULL; lLines++)
		{
		iLen = strlen (sz);

		/* Squeeze blank lines if needed */
		if (iBlank)
			if (blank (sz))
				continue;

		/* Number the output if needed */
		if (iNum)
			{
			printf ("%d:", lLines);
			}

		/* Print specials if needed */
		for (i = 0; i < (iLen); i++)
			{

		/* Convert to upper or lower */
			if (iUpc)
				{
				sz[i] = toupper (sz[i]);
				}
			else if (iDownc)
				{
				sz[i] = tolower (sz[i]);
				}
			if ((iscntrl (sz[i])) && (iCtrl))
				{
				printf ("^%c", sz[i] + 64);
				}
			else if ((sz[i] == '\t') && (iTabs))
				{
				printf ("^%c", sz[i] + 64);
				}
			else
				{
				printf ("%c", sz[i]);
				}
			}

		/* Print the EOL if needed */
		if (iEnds)
			printf ("$");

		if (iPage)
			pagel (NULL);
		else
			printf ("\n");

		}
	return;
	}

int blank (char buf[MAX_STR])

	{
	int i;

	for (i = 0; ((i <= MAX_STR) && (buf[i] != NULL)); i++)
		if (!isspace(buf[i]))
			return (FALSE);

	return (TRUE);
	}

void usage (void)
				     	 	 	 
	{
	printp ("TYPE", "Displays a text file or files");
	printu ("TYPE", "[/N] [/S] [/C] [/E] [/T] [/U] [/L] [/A] [file..]");
	printo ("N", "Number the output, starting with 1");
	printo ("S", "Squeeze together non-blank lines");
	printo ("C", "Show control characters using \'^\'");
	printo ("E", "Show the ends of lines using \'$\'");
	printo ("T", "Show tabs using \'^I\'");
	printo ("U", "Convert file to uppercase");
	printo ("L", "Convert file to lowercase");
	printo ("A", "Show everything (same as /N /C /E /T)");
	exit (1);
	}
