/* $Id: msglib.h 1.3 1995/05/26 20:29:24 ska Exp ska $
   $Locker: ska $	$Name:  $	$State: Exp $

	Global message handling definitions.

   $Log: msglib.h $
   Revision 1.3  1995/05/26 20:29:24  ska
   add: access to application's name.
   add: prototype of fputmc().

   Revision 1.2  1995/04/21 04:35:46  ska
   version 1.0

   Revision 1.1  1995/04/21 04:11:29  ska
   Initial revision

*/

#ifndef __MSGLIB_H
#define __MSGLIB_H

#include <stdio.h>
#include <portable.h>


#ifdef _MICROC_	/* does not support typedef's */
#define MSGID unsigned char*
#else
typedef const unsigned char *MSGID;
#endif
/* DCL_MSG declares one MeSsaGe IDentifier;
   XDCL_MSG may be used while implementing the entire application,
   because it can bypass the message compiler by staticly coding 
   the local messages */
#define DCL_MSG(msg) extern const unsigned char msg[]
#ifdef DEFINE_MESSAGES
#define XDCL_MSG(msg,str) const unsigned char msg[] = str
#else
#define XDCL_MSG(msg,str) DCL_MSG(msg)
#endif

#define msgLock(msg) (msg)
#define msgRetreive(msg) (msg)
#define msgUnlock(msg)
#define msgUnlockString(msg)
/* this needs the one-byte alignment within the error definition files */
#define msgErrorNumber(msg) ((msg)[-1]) 
/* msgInit() is a function */

#define ANSI_TEST 0		/* check if output stream is connected to a CON device */
#define ANSI_DENY 1		/* do no send ANSI sequences into the output stream */
#define ANSI_ALLOW 2	/* do not check for CON device, send sequences */

#define NOISE_TEST 2	/* check if stdin is redirected, before emitting interactive messages.
						   skip if not interactive message */
#define NOISE_DENY 1	/* skip messages t all */
#define NOISE_ALLOW 0	/* emit messges */
#define NOISE_ERR (-1)	/* error, returned by getNoiseLevel() on error */

#define ENoise_interactive 0	/* must be number zero !!! */
#define ENoise_informative 1
#define ENoise_fatal 2
#define ENoise_error 3
#define ENoise_warning 4
#define ENoise_none 5
#define	ENoise_Nr 6	/* number of noise levels */

/* uncomment next #define so any device (including LPT:, PRN:, NUL:, COM:
   AUX:, CLOCK$, MSCD001, etc.) will be treated as consoles (not redirected).

   Note: message library must be rebuilt! */
/* #define USE_ISADEVICE */



void msgInit(void);
/* Initialize the message system.

   Currently:
   	+ Check for ANSI and set the ANSI test flag to ANSI_TEST or ANSI_DENY.
   	+ Check for redirected stdin and set the interactive test flag to
   	  NOISE_ALLOW or NOISE_DENY.
   	+ Determine the application's name.
*/

char *appName(void);
/* Return the name of the current process. This name is upper-cased
   and without drive, path, and extension.
*/


int fputmc(int c, int rep, FILE *fout);
/* Dump the character rep times into the stream fout.

	Return:		 0: OK
				!0: failure
*/

int skprintf(FILE *fout, const char *fmt, const unsigned char *args);
/* input: fout: output stream
          fmt: format string
          args: pointer to the first argument

   output: number of emitted characters

   This function uses the isatty() function on the stream fout to
   determine, if to emit the ANSI enclosures or not. That behaviour is
   controlled by the ANSI test flag.

*/


#ifndef _MICROC_
int fd_fprintf(FILE *fp, const char *fmt, ...);
#else
register int fd_fprintf(FILE *fp, const char *fmt);
#endif
/* Wrapper for skprintf().

   This functions allows the usage of skprintf() with the syntax of
   fprintf().
*/


void setAnsiTest(int flag);
/* Function to set the ANSI test flag. 

   Should be called before msgInit() to skip the check, if an ANSI
   driver is present. This test will set the flag to ANSI_TEST if a
   driver is present and to ANSI_DENY if none is found.

   In the functions using this flag, ANSI_TEST means, that ANSI will
   be used, if the file stream is connected to a console (CON:).

   May be called anytime the application wants to alter the ANSI test
   behaviour of skprintf(). To retrieve the current state of the flg
   use getAnsiTest().

   flag should be one of ANSI_????, otherwise the behaviour of all
   functions using this flag is undefined.

   Initial setting of the flag: ANSI_TEST.

*/

int getAnsiTest(void);
/* Function retrieves the current state of the ANSI test flag
*/

void setNoiseLevel(unsigned noiseLevel, int flag);
/* Function to set the allow/disallow flag for each noise level.

   May be called anytime the application wants to alter the interactive
   test behaviour of the interactive() function. To retrieve the current
   state of the flag use getNoiseLevel().

   flag should be one of NOISE_???, otherwise the behaviour of all
   functions using this flag is undefined.

   Initial setting is NOISE_ALLOW.

	For ENoise_interactive only:

   Should be called after msgInit() to ensure, that the test is
   performed anytime an interactive message is emitted.

   Initial setting of the ENoise_interactive flag: NOISE_TEST, but is
   altered to either NOISE_DENY or NOISE_ALLOW by the msgInit()
   function.

*/

int getNoiseLevel(unsigned noiseLevel);
/* Function retrieves the current state of the allow/disallow flag
   of the specified noise level.
*/



/* Message emitting functions:

   hlpScreen:
   	Emit the message E_hlpScreen onto stdout and exit with errorlevel.
   error:
   	Emit the supplied message onto stderr and exit with errorlevel.
   	Preceed the message with the application name and E_error.
   fatal:
   	Equals to error, but preceed with the application name and
   	E_fatal.
   warning:
   	Emit the supplied message onto stderr and preceed the message
   	with the application name and E_warning.
   informative:
   	Emit the supplied message onto stderr and preceed the message
   	with the application name.
   interactive:
   	Emit the supplied message onto stdout. If redirection test is 
   	enabled, the message is not emitted, if stdin is redirected to
   	a non-CON: device (or, by switch, to a file).
   message (none):
   	Emit the supplied message onto the supplied file stream.
*/

#ifdef _MICROC_
register void error(MSGID msg);
register void warning(MSGID msg); 
register void fatal(MSGID msg);
register void informative(MSGID msg);
register void interactive(MSGID msg);
register void message(FILE *fp, MSGID msg);

#else /* no Micro-C */
void error(MSGID msg,...); 
void warning(MSGID msg,...); 
void fatal(MSGID msg,...);
void informative(MSGID msg,...);
void interactive(MSGID msg,...);
void message(FILE *fp, MSGID msg,...);

#endif

void hlpScreen(void);
    DCL_MSG(E_hlpScreen);	/* every application must define a help screen */
/* whitespaces prevent, that the msgComp cut this declaration out */

/* no format string error reporting
#define Nr_E_hlpFmtStr 20
    DCL_MSG(E_hlpFmtStr);
*/



DCL_MSG(I_white);
DCL_MSG(I_cls);
DCL_MSG(I_clreol);
DCL_MSG(I_home);
DCL_MSG(I_up);
DCL_MSG(I_down);
DCL_MSG(I_left);
DCL_MSG(I_right);
DCL_MSG(I_goto);
DCL_MSG(I_save);
DCL_MSG(I_restore);
DCL_MSG(I_bright);
DCL_MSG(I_blink);
DCL_MSG(I_inverse);
DCL_MSG(I_underline);
DCL_MSG(I_hidden);
DCL_MSG(I_normal);
DCL_MSG(I_black);
DCL_MSG(I_red);
DCL_MSG(I_green);
DCL_MSG(I_yellow);
DCL_MSG(I_blue);
DCL_MSG(I_magenta);
DCL_MSG(I_cyan);
DCL_MSG(E_UserBreak);
DCL_MSG(E_internalFailure);
DCL_MSG(E_noMem);
DCL_MSG(E_readFile);
DCL_MSG(E_option);
DCL_MSG(E_openFile);
DCL_MSG(E_writeFile);
DCL_MSG(I_warning);
DCL_MSG(I_error);
DCL_MSG(I_fatal);
DCL_MSG(I_appName);
DCL_MSG(I_noise);
DCL_MSG(E_missingCmdLineArg);
DCL_MSG(E_cmdLineArg);

#endif
