/*
    Source of the message handling library
    Copyright (C) 1995  Steffen Kaiser

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
   
*/
/* $RCSfile: msgemit.c $
   $Locker: ska $	$Name: version2 $	$State: Exp $

   Fehlerbehandlungsroutinen

   $Log: msgemit.c $
   Revision 1.1  1995/05/26 22:53:58  ska
   Initial revision

   Revision 1.5  1995/05/26 20:25:12  ska
   add: read-only access to the application's name (created by msgInit()).

   Revision 1.4  1995/04/21 04:35:46  ska
   version 1.0

   Revision 1.3  1995/04/21 04:11:29  ska
   chg: directory structure.

   Revision 1.2  1995/04/19 09:06:46  ska
   chg: multiple message emitting functions
   add: message pre/post-process
   chg: restructure of BCC/Micro-C support

   Revision 1.1  1995/04/10 23:24:39  ska
   Initial revision

*/

#include <stdio.h>
#ifndef _MICROC_
#include <string.h>
#include <process.h>
#include <dos.h>
#endif
#include <freedos.h>
#include <msglib.h>

#ifdef USE_ISADEVICE
#define isatty isadev
#endif

#define NUL '\0'

#ifdef __TURBOC__
#pragma hdrstop
#endif

/* #define ERROR_MAKING_STATE  */ /* Fehlertexte erzeugen */
/* #include "yerror.h" */

#ifndef lint
static char const rcsid[] = 
	"$Id: msgemit.c 1.1 1995/05/26 22:53:58 ska Exp ska $";
#endif

/* no format string error reporting
const unsigned char E_hlpFmtStr[] = "Error in format string near '%s'";
*/


static unsigned char noiseDisallow[ENoise_Nr] = {NOISE_TEST};
/* The rest of the array is filled with zeros meaning: allowed */




#ifdef MAXFILE
	static char XappName[MAXFILE] = "";
#define maxName MAXFILE
#else
	static char XappName[9] = "";
#define maxName 9
#endif
char *appName(void)
{	return XappName;	}


void __msgOutput();

#ifdef _MICROC_
#include <file.h>	/* for fileno() */

/* Micro-C's K&R-style / ** / is the equivalent for ANSI-C's ## */
	/* need std stack frame */
/* #define msgFct(a) register void a(MSGID msg)								\*/ 
/*	asm {																	\
		mov cx, ENoise_/**/a												\
  		jmp short __warning_emit_msg										\
	} */

/* Above Example does not work with Micro-C, because it has to be processed
   twice by the preprocessor. Therefore use precalculated noise level. */

#define msgFct(a,b) register void a(MSGID msg)	/* need std stack frame */	\
	asm {																	\
		mov cx, b															\
  		jmp short __warning_emit_msg										\
	}


register void interactive(MSGID msg)
/* interactive means, that the function should be checked, wether or not
   stdin is redirected. If stdin is redirected this message is ignored */
{	asm "push ax";	/* preserve nargs */
	if(noiseDisallow[ENoise_interactive] == NOISE_TEST) {
		if(!isatty(fileno(stdin))) {
			asm "pop ax";
			return;
		}
	}
	/* else the application disallowed the test */

	asm {
		pop ax
		mov cx, ENoise_interactive
__warning_emit_msg:
		shl ax, 1	; amount of bytes used by all arguments
		lea bx, 2[bp]	; address below last arg
		add ax, bx	; address of format string
		push ax		; __msgOutput()''s first arg
		push cx		; __msgOutput()''s second arg
		push ax		; dummy argument (unused fout)
		call ___msgOutput
		add sp, 6
	}
}

register void message(FILE *fp, MSGID msg)	/* need two args */
{
	asm {
		shl ax, 1	; amount of bytes used by all arguments
		lea bx, 2[bp]	; address below last arg
		add ax, bx	; address of file pointer
		mov bx, ax
		dec ax
		dec ax		; address of format string
		push ax		; __msgOutput()''s first arg	(fmt str)
		mov ax, ENoise_none	; no immediate form tells ASM here
		push ax		; __msgOutput()''s second arg	(noise level)
		push Word Ptr [bx]	; file pointer
		call ___msgOutput
		add sp, 6
	}
}

#else	/* no Micro-C */

/* I don't care about about the waste of mem here; it's portable and
   not very overweighted.
*/
/* #define msgFct(a) void a(MSGID msg, ...)							\
	{	__msgOutput(&msg, ENoise_##a); } */
#define msgFct(a,b) void a(MSGID msg, ...)							\
	{	__msgOutput(&msg, b); 	}

void interactive(MSGID msg, ...)
/* interactive means, that the function should be checked, wether or not
   stdin is redirected. If stdin is redirected this message is ignored */
{	if(noiseDisallow[ENoise_interactive] == NOISE_TEST) {
		if(!isatty(fileno(stdin)))
			return;
	}
	/* else the application disallowed the test */

	__msgOutput(&msg, ENoise_interactive);
}

void message(FILE *fp, MSGID msg, ...)
{	__msgOutput(&msg, ENoise_none, fp);	}

#endif	/* if Micro-C */

/* declare *normal* message wrapper functions */
msgFct(error, ENoise_error)
msgFct(fatal, ENoise_fatal)
msgFct(warning, ENoise_warning)
msgFct(informative, ENoise_informative)


/* Micro-C pushes parameters left->right; therefore right parameters
   cannot be omitted by default to access the leftmost parameters. */

/* To place the function definition behind its usage allows ANSI-C
   compilers to stuff as many argument into the function as supplied 
   on function call (less than three!). */
#ifdef _MICROC_
void __msgOutput(const MSGID *arg, int noiseLevel, FILE *fout)
#else
void __msgOutput(const MSGID *arg, int noiseLevel, FILE *fp)
#endif
/* emit the message: 
	fout: file stream to emit into
	*arg: message ID
	arg[1..]: arguments of the message
	noiseLevel: noise level according to ENoise_????
	fp: output file stream; for ENoise_none only!
*/
{	MSGID noiseMsg;
#ifndef _MICROC_	/* Under Micro-C this function is always called with three
					   parameters. */
	FILE *fout;			/* because fp exists only for ENoise_none */
#endif

	/* Check if message is allowed to be emitted */
	if(noiseDisallow[noiseLevel])
		return;

/* nxtArg() returns the address of the successor */
#ifndef _MICROC_	/* 2 arg has greater address */
#define nxtArg(arg) (unsigned char*)(arg + 1)
#else	/* 2 arg has lesser address */
#define nxtArg(arg) (arg - 1 - 1 /* Micro-C won't sub 2 here */)
#endif

	/* Prepare message emitting */
	switch(noiseLevel) {
		case ENoise_informative: noiseMsg = NULL; goto emitHead;
		case ENoise_warning: noiseMsg = I_warning; goto emitHead;
		case ENoise_error: noiseMsg = I_error; goto emitHead;
		case ENoise_fatal: noiseMsg = I_fatal; 
		emitHead:
/* except ENoise_none and ENoise_interactive, all messages are preceeded by:
	<application_name>: <warning|error|fatal>:
*/
			fout = stderr;	/* all those noise levels go to stderr !!! */
			message(stderr, I_appName, XappName);
			if(noiseMsg) {
				message(stderr, I_noise, msgLock(noiseMsg));
				msgUnlock(noiseMsg);
			}
			break;

		case ENoise_interactive: fout = stdout; break;

#ifndef _MICROC_		/* under Micro-C no fp needed */
		case ENoise_none: fout = fp; break;	/* file pointer supplied
			on function call */
#endif
	}

	/* message emitting */
	skprintf(fout, msgLock(*arg), nxtArg(arg));
	msgUnlock(*arg);

	/* Finish message emitting */
	switch(noiseLevel) {
		case ENoise_warning:
		case ENoise_informative:
			fputs(".\n", fout);	/* close message */
			break;
		case ENoise_error:
		case ENoise_fatal:
			fputs(".\n", fout);	/* close message */
			exit(msgErrorNumber(*arg));
		/* case ENoise_none:
		   case ENoise_interactive:
		   		These are programmer defined messages, so leave them alone.
		*/
	}
}

void setNoiseLevel(unsigned noiseLevel, int flag)
/* set the interactive test flag to flag */
{	if(noiseLevel < ENoise_Nr)
		noiseDisallow[noiseLevel] = flag; 
}

int getNoiseLevel(unsigned noiseLevel)
/* get the interactive test flag to flag */
{	return noiseLevel < ENoise_Nr? noiseDisallow[noiseLevel]: NOISE_ERR; 
}


void msgInit(void)
/* Initialize message system */
{	char *p;
	int i;
	USEREGS

/*	Retrieve the application's name */
	if((p = strrchr(_argv[0], '\\')) == NULL
	  && (p = strrchr(_argv[0], ':')) == NULL)
	  	p = _argv[0] - 1;

	for(i = 0; i < (maxName-1) && *++p != '.' && *p;)
		XappName[i++] = *p;
	XappName[i] = NUL;


/*	Check for redirected stdin */
	if(noiseDisallow[ENoise_interactive] == NOISE_TEST) {
		noiseDisallow[ENoise_interactive] = isatty(fileno(stdin))?
			NOISE_ALLOW: NOISE_DENY;
	}

/*	Check for ANSI;; very simple, will fail before ANSI v4 */
	if(getAnsiTest() == ANSI_TEST) {
		_AX = 0x1a00;	/* multiplex installation test for ANSI driver */
		geninterrupt(0x2f);
		if(!_AL)
			setAnsiTest(ANSI_DENY);
	}
}

